#include "Camera3D.h"

using namespace glm;

const float Camera3D::kMinFocusDistance_ = 0.1f;

Camera3D::Camera3D() : Frame3D(), focusDistance_(0.0f) {}

Camera3D::~Camera3D() {}

Camera3D::Camera3D(const vec3& origin, const vec3& up, const vec3& forward)
  : Frame3D(origin, up, forward), focusDistance_(length(forward))
{
}

void Camera3D::LookAt(float eyeX, float eyeY, float eyeZ,
                      float centerX, float centerY, float centerZ,
                      float upX, float upY, float upZ)
{
    origin_ = vec3(eyeX, eyeY, eyeZ);
    vec3 forward = vec3(centerX, centerY, centerZ) - origin_;
    forward_ = normalize(forward);
    vec3 right = cross(forward_, vec3(upX, upY, upZ));
    up_ = normalize(cross(right, forward_));
    focusDistance_ = length(forward);
}

void Camera3D::MoveForward(float d) {
    float newFocusDistance = focusDistance_ - d;
    if (newFocusDistance < kMinFocusDistance_)
        d = focusDistance_ - kMinFocusDistance_;
    Frame3D::MoveForward(d);
    focusDistance_ -= d;
}

void Camera3D::RotateX(float theta) {
    vec3 focus = origin_ + focusDistance_ * forward_;
    vec3 right = normalize(cross(forward_, up_));
    RotateWorld(theta, focus, right);
}

void Camera3D::RotateY(float theta) {
    vec3 focus = origin_ + focusDistance_ * forward_;
    RotateWorld(theta, focus, vec3(0.0f, 1.0f, 0.0f));
}