// This file contains common stuffs used throughout the project.

#include <sstream>

#ifndef COMMON_H__
#define COMMON_H__

#include <iostream>
#include <Eigen/Dense>

#define FOR_EACH(item, begin, end) \
    for (auto item = begin; item != end; ++item)
#define FOR(item, collection) \
    for (auto item = collection->Begin(); item != collection->End(); ++item)
#define FOR_CONST(item, collection) \
    for (auto item = collection->CBegin(); item != collection->CEnd(); ++item)

// Some classes have empty default constructor and destructor. We use this macro
// to save some typing.
#define DUMMY_DEFAULT_CONSTRUCTOR_AND_DESTRUCTOR(TypeName) \
    TypeName() {}; \
    virtual ~TypeName() {};

// The default C++ assert does not support message printing. This macro supports
// that.
#ifndef NDEBUG
#   define ASSERT(condition, message) \
    do { \
        if (!(condition)) { \
            std::cerr << "Assertion `" #condition "` failed in " << __FILE__ \
                      << " line " << __LINE__ << ": " << message << std::endl; \
            std::exit(EXIT_FAILURE); \
        } \
    } while (false)
#else
#   define ASSERT(condition, message) do { } while (false)
#endif

// Some common constants.
const float kDegreeToRadian_ = 0.0174533f;
const float kRadianToDegree_ = 57.295780f;
const float kPi_ = 3.1415927f;

// Frequently used Eigen stuffs.
#define kVecUnitX_ Eigen::Vector3f::UnitX()
#define kVecUnitY_ Eigen::Vector3f::UnitY()
#define kVecUnitZ_ Eigen::Vector3f::UnitZ()
#define kVecZero_  Eigen::Vector3f::Zero()
#define kQuatIdentity_ Eigen::Quaternionf::Identity()

// Converts an integer to a string
inline std::string IntToString(int n)
{
    std::stringstream myStream;
    myStream << n;
    return myStream.str();
}

// Converts a string to an integer
inline int StringToInt(const std::string& s)
{
    std::stringstream myStream(s);
    int n;
    myStream >> n;
    return n;
}

#endif    // COMMON_H__