#include "Effector.h"

using namespace std;
using namespace Eigen;

Effector::Effector(const string &name, const Joint *joint, float ratio)
  : name_(name)
{
    auto parent = joint->GetParent();
    if (parent != nullptr) { // not the root
        parent_ = parent;
        localPosition_ = joint->GetLocalPosition() * ratio;
    } else { // this effector is attached to the root, ignore ratio
        parent_ = joint;
        localPosition_ = kVecZero_;
    }

    ComputeGlobalPosition();
    globalTargetPosition_ = globalPosition_;
}

const Joint* Effector::GetParent() const { return parent_; }
const string Effector::GetName() const { return name_; }

Vector3f Effector::GetTargetPosition() const { return globalTargetPosition_; }

Vector3f Effector::GetLocalPosition() const { return localPosition_; }

Vector3f Effector::GetGlobalPosition() const { return globalPosition_; }

void Effector::ComputeGlobalPosition() {
    globalPosition_ =
        parent_->GetGlobalQuaternion()._transformVector(localPosition_) +
        parent_->GetGlobalPosition();
}

void Effector::UpdateTarget(float newX, float newY, float newZ) {
    globalTargetPosition_ = Vector3f(newX, newY, newZ);
}