#include "EffectorCollection.h"

using namespace boost;
using namespace std;

int EffectorCollection::GetEffectorCount() const { return effectors_.size(); }

void EffectorCollection::AddEffector(const Tree* tree, const string& name,
                                     const string& jointName, float ratio)
{
    auto joint = tree->GetJoint(jointName);
    ASSERT(joint != nullptr, "No such joint exists.");
    ASSERT(effectors_.count(name) == 0, "Duplicate effector's names.");
    effectors_[name] = Effector(name, joint, ratio);
}

void EffectorCollection::UpdateTarget(const string& name,
                          float newX, float newY, float newZ)
{
    ASSERT(effectors_.count(name) > 0, "No such effector exists.");
    effectors_[name].UpdateTarget(newX, newY, newZ);
}

void EffectorCollection::ComputeGlobalData() {
    for_each(begin(effectors_), end(effectors_), [&] (StrEffPair &pair) {
        pair.second.ComputeGlobalPosition();
    });
}

Effector& GetValue(pair<const string, Effector>& pair) { return pair.second; }

const Effector& GetValueConst(const pair<const string, Effector>& pair) {
    return pair.second;
}

EffectorIterator EffectorCollection::Begin() {
    return make_transform_iterator(effectors_.begin(), GetValue);
}

EffectorIterator EffectorCollection::End() {
    return make_transform_iterator(effectors_.end(), GetValue);
}

EffectorIteratorConst EffectorCollection::CBegin() const {
    return make_transform_iterator(effectors_.cbegin(), GetValueConst);
}

EffectorIteratorConst EffectorCollection::CEnd() const {
    return make_transform_iterator(effectors_.cend(), GetValueConst);
}