#include "Frame3D.h"
#include <glm/ext.hpp>

using namespace glm;

Frame3D::Frame3D()
  : origin_(vec3(0.0f)),
    forward_(vec3(0.0f, 0.0f, -1.0f)),
    up_(vec3(0.0f, 1.0f, 0.0f))
{
}

Frame3D::~Frame3D() {}

Frame3D::Frame3D(const vec3& origin, const vec3& up, const vec3& forward) {
    Set(origin, up, forward);
}

void Frame3D::Set(const vec3& origin, const vec3& up, const vec3& forward) {
    origin_ = origin;
    forward_ = normalize(forward);
    vec3 right = cross(forward, up);
    vec3 realUp = cross(right, forward);
    up_ = normalize(realUp);
}

void Frame3D::BuildLocalToWorldMatrix() {
    vec3 right = normalize(cross(forward_, up_));
    matLocalWorld_ = mat4(mat4x3(right, up_, -forward_, origin_));
}

void Frame3D::BuildWorldToLocalMatrix() {
    vec3 right = normalize(cross(forward_, up_));
    mat4 R(transpose(mat3(right, up_, -forward_)));
    matWorldLocal_ = translate(R, -origin_);
}

mat4 Frame3D::GetLocalToWorldMatrix() const { return matLocalWorld_; }

mat4 Frame3D::GetWorldToLocalMatrix() const { return matWorldLocal_; }

void Frame3D::MoveForward(float d) { origin_ += forward_ * d; }

void Frame3D::MoveRight(float d) {
  vec3 right = cross(forward_, up_);
  origin_ += normalize(right) * d;
}

void Frame3D::MoveUp(float d) { origin_ += up_ * d; }

void Frame3D::RotateWorld(float theta, const vec3& p, const vec3& v) {
    // create the transformation matrix by first translating to p, rotate around
    // v, and translate back
    mat4 T = translate(mat4(1.0f), p);
    mat4 TR = rotate(T, theta, v);
    mat4 M = translate(TR, -p);
    // rotate the origin, up, and forward vectors
    origin_ = (M * vec4(origin_, 1.0f)).xyz;
    up_ = (normalize(M * vec4(up_, 0.0f))).xyz;
    forward_ = (normalize(M * vec4(forward_, 0.0f))).xyz;
}