#include "FrameRate.h"
#include "GLFW/glfw.h"

FrameRate::~FrameRate() {}

FrameRate::FrameRate(int averageFrameCount)
  : avgFrameCount_(averageFrameCount),
    frameCount_(0),
    frameIndex_(0),
    lastFrameRate_(0.0)
{ timeArray_.resize(avgFrameCount_); }

void FrameRate::StartFrame() {
    if (frameCount_ < avgFrameCount_) // we're in the first few frames
        ++frameCount_;

    timeArray_[frameIndex_] = glfwGetTime();
    ++frameIndex_;
    frameIndex_ %= avgFrameCount_;
}

void FrameRate::EndFrame() {
    if (frameCount_ <= 0) // shouldn't happen
        return;

    int index = (frameCount_ == avgFrameCount_) ? frameIndex_ : 0;
    double currentTime = glfwGetTime();
    lastFrameRate_ = frameCount_ / (currentTime - timeArray_[index]);
}

double FrameRate::GetFrameRate() const { return lastFrameRate_; }