#ifndef FRAME_RATE_H__
#define FRAME_RATE_H__

#include <vector>

// This class can be used to measure the frame-rate of a program in terms of
// frames-per-second (fps).
class FrameRate {
public:
    FrameRate(int averageFrameCount = kDefaultAverageFrameCount_);
    virtual ~FrameRate();

    // Starts measuring. To be called at the beginning of a frame.
    void StartFrame();
    // Stops measuring. To be called at the end of a frame.
    void EndFrame();
    // Returns the frame-rate averaged over averageFrameCount frames.
    double GetFrameRate() const;

private:
    // The default number of frames to average over.
    static const int kDefaultAverageFrameCount_ = 5;

    // The number of frames we average over. We don't measure fps every frame
    // since it's noisy.
    int avgFrameCount_;
    // The current number of frames since the first call to StartFrame().
    // It stops increasing once it reaches avgFrameCount_.
    int frameCount_;
    // The index of
    int frameIndex_;
    // We store the timings measured in this vector.
    std::vector<double> timeArray_;
    // The last measured fps.
    double lastFrameRate_;
};

#endif    // FRAME_RATE_H__