#include <functional>
#include "AntTweakBar/AntTweakBar.h"
#include "IKProgram.h"
#include "IKSolverDLS.h"

using namespace std;

IKProgram::IKProgram() {}
IKProgram::~IKProgram() {}
IKProgram::IKProgram(Data *data) :
    Program(data),
    iterationCount_(30),
    firstTime_(true),
    step_(0)
{
}

void TW_CALL IKProgram::GetIterationCount(void *value, void *data)
{
    IKProgram *program = static_cast<IKProgram *>(data);
    int *iterationCount = static_cast<int *>(value);
    *iterationCount = program->iterationCount_;
}

void TW_CALL IKProgram::SetIterationCount(const void *value, void *data)
{
    IKProgram *program = static_cast<IKProgram *>(data);
    const int *iterationCount = static_cast<const int *>(value);
    program->iterationCount_ = *iterationCount;
}

void TW_CALL IKProgram::SetDLSLambda(const void *value, void *data)
{
    IKProgram *program = static_cast<IKProgram *>(data);
    const float *lambda = static_cast<const float *>(value);
    IKSolverDLS *solver = dynamic_cast<IKSolverDLS *>(program->solver_.get());
    if (solver) {
        solver->SetLambda(*lambda);
    }
}

void TW_CALL IKProgram::GetDLSLambda(void *value, void *data)
{
    IKProgram *program = static_cast<IKProgram *>(data);
    float *lambda = static_cast<float *>(value);
    IKSolverDLS *solver = dynamic_cast<IKSolverDLS *>(program->solver_.get());
    if (solver) {
        *lambda = solver->GetLambda();
    }
}

void TW_CALL IKProgram::SetDrawLocalFrames(const void *value, void *data)
{
    IKProgram *program = static_cast<IKProgram *>(data);
    const bool *isLocalFramesEnabled = static_cast<const bool *>(value);
    Renderer *renderer = program->renderer_.get();
    if (renderer) {
        if (*isLocalFramesEnabled) {
            renderer->Enable(Renderer::LOCAL_FRAME);
        } else {
            renderer->Disable(Renderer::LOCAL_FRAME);
        }
    }
}

void TW_CALL IKProgram::GetDrawLocalFrames(void *value, void *data)
{
    IKProgram *program = static_cast<IKProgram *>(data);
    bool *isLocalFramesEnabled = static_cast<bool *>(value);
    Renderer *renderer = program->renderer_.get();
    if (renderer) {
        *isLocalFramesEnabled = renderer->IsEnable(Renderer::LOCAL_FRAME);
    }
}

void IKProgram::CreateBar()
{
    if (!firstTime_) {
        return;
    }

    TwAddVarCB(data_->bar_, "Draw local frames", TW_TYPE_BOOLCPP,
        &IKProgram::SetDrawLocalFrames, &IKProgram::GetDrawLocalFrames, this,
        " label='Draw local frames' ");

    // add iteration count control
    TwAddVarCB(data_->bar_, "Iteration count", TW_TYPE_INT32,
        &IKProgram::SetIterationCount, &IKProgram::GetIterationCount, this,
        " min=1 max=50 step=1 label='Iteration count' ");

    switch (method_) {
    case IKSolver::DLS:
        TwAddVarCB(data_->bar_, "Lambda", TW_TYPE_FLOAT, &IKProgram::SetDLSLambda, &IKProgram::GetDLSLambda, this,
            " min=0.1 max=1.0 step=0.1 label='Lambda' ");
        break;
    }

    firstTime_ = false;
}

void IKProgram::Draw()
{
    renderer_->DrawFloor(50.0f);
    renderer_->DrawTree(tree_.get());
    renderer_->DrawTargets(effectors_.get());
    renderer_->DrawGlobalAxes(50.0f);

    CreateBar();
}

Renderer* IKProgram::GetRenderer() { return renderer_.get(); }

void IKProgram::BeginFrame()
{
}

void IKProgram::ResetTree() { tree_.reset(new Tree()); }

void IKProgram::ResetEffectors() {
    effectors_.reset(new EffectorCollection());
    tree_->ComputeGlobalData();
}

void IKProgram::ResetRenderer() {
    renderer_.reset(new Renderer(data_->camera_));
    renderer_->SetCameraLookAt(0.0f, 4.0f, 4.0f,
                               0.0f, 0.0f, 0.0f,
                               0.0f, 1.0f, 0.0f);
}

void IKProgram::ResetSolver(IKSolver::Method newMethod) {
    method_ = newMethod;
    switch (newMethod) {
    case IKSolver::DLS:
        solver_.reset(new IKSolverDLS(tree_.get(), effectors_.get()));
        break;
    case IKSolver::SDLS:
        // TODO: implement SDSL
        break;
    }
}