#ifndef IK_PROGRAM_H__
#define IK_PROGRAM_H__

#include <string>
#include "Program.h"
#include "Tree.h"
#include "EffectorCollection.h"
#include "IKSolver.h"
#include "Renderer.h"

/* An IKProgram extends a Program to provide a default template for all IK tests */
class IKProgram : public Program {
public:
    IKProgram();
    IKProgram(Data *data);
    virtual ~IKProgram();

    virtual void Update(float time) = 0;
    virtual void Draw() override;
    virtual void BeginFrame() override;

    void ResetTree();
    void ResetEffectors();
    void ResetRenderer();
    void ResetSolver(IKSolver::Method newMethod);

    Renderer* GetRenderer();

    static void TW_CALL SetDrawLocalFrames(const void *value, void *data);
    static void TW_CALL GetDrawLocalFrames(void *value, void *data);
    static void TW_CALL SetIterationCount(const void *value, void *data);
    static void TW_CALL GetIterationCount(void *value, void *data);
    static void TW_CALL SetDLSLambda(const void *value, void *data);
    static void TW_CALL GetDLSLambda(void *value, void *data);

protected:
    IKSolver::Method method_;
    std::unique_ptr<Tree> tree_;
    std::unique_ptr<EffectorCollection> effectors_;
    std::unique_ptr<IKSolver> solver_;
    std::unique_ptr<Renderer> renderer_;

    int iterationCount_;

    bool firstTime_;

    int step_;
    static const int kMaxStep_ = 180;

    void CreateBar();
    void SetDLSLambda(float value);
    float GetDLSLambda();

};

#endif    // IK_PROGRAM_H__