// TODO: choose lambda automatically

#include "IKSolverDLS.h"

using namespace Eigen;
using namespace std;

const float IKSolverDLS::kDefaultLambda_ = 0.4f;

IKSolverDLS::IKSolverDLS(Tree* tree, EffectorCollection *effectors, float lambda) :
    IKSolverJacobian(tree, effectors), lambda_(lambda) {}

float IKSolverDLS::GetLambda() const
{
    return lambda_;
}

void IKSolverDLS::SetLambda(float newLambda)
{
    lambda_ = newLambda;
}

void IKSolverDLS::ComputeDeltaTheta()
{
    MatrixXf u = J_ * J_.transpose();
    u = u + u.Identity(u.rows(), u.cols()) * lambda_ * lambda_;
    VectorXf dT = u.colPivHouseholderQr().solve(eClamped_);
    //VectorXf dT = u.householderQr().solve(eClamped_);
    DeltaTheta_ = J_.transpose() * dT;

    // scale the delta thetas for the angle segment (all except the first 3 elements)
    auto dThetaAnglePart = DeltaTheta_.segment(3, DeltaTheta_.size() - 3);
    float maxAngleChange = max(fabs(dThetaAnglePart.maxCoeff()), fabs(dThetaAnglePart.minCoeff()));
    if (maxAngleChange > maxAngleChange_) {
        dThetaAnglePart *= maxAngleChange_ / maxAngleChange;
    }
}