#ifndef IKSOLVER_JACOBIAN_H__
#define IKSOLVER_JACOBIAN_H__

#include "Eigen/Dense"
#include "EffectorCollection.h"
#include "IKSolver.h"

class IKSolverJacobian : public IKSolver {
public:
    DUMMY_DEFAULT_CONSTRUCTOR_AND_DESTRUCTOR(IKSolverJacobian)

    IKSolverJacobian(Tree* tree, EffectorCollection* effectors);

    void BuildJacobian(const Tree* tree, const EffectorCollection* effectors);
    void Solve(Tree* tree, EffectorCollection* effectors,
               int iterationCount = kDefaultIterationCount_) override;

protected:
    Eigen::MatrixXf J_;
    std::map<int, int> indexMap_;

    float DMax_;
    Eigen::VectorXf e_;
    Eigen::VectorXf eClamped_;
    Eigen::VectorXf DMaxi_;
    Eigen::VectorXf di_;
    Eigen::VectorXf DeltaTheta_;

    void ComputeE(const EffectorCollection* effectors);
    virtual void ComputeDeltaTheta() = 0;
    void UpdateThetas(Tree* tree, EffectorCollection* effectors);
    void ComputeDMaxi(const EffectorCollection* effectors);
};

#endif    // IK_SOLVER_JACOBIAN_H__