#include "EffectorOnBone.h"
#include "Eigen/Dense"

using namespace std;
using namespace Eigen;

EffectorOnBone::EffectorOnBone()
{
}

EffectorOnBone::~EffectorOnBone()
{
}

EffectorOnBone::EffectorOnBone(Data *data) :
    IKProgram(data)
{
    name_ = "Effector on bone";
    description_ = "This IK program tests a tree with an effector defined in the middle of a bone.";

    // build the tree
    ResetTree();
    tree_->AddJoint("joint0", 0.08f, "", Vector3f(0.0f, 0.5f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitY_, 0.0f, kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint1", 0.08f, "joint0", Vector3f(0.0f, 2.0f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitY_, 0.0f, kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint2", 0.08f, "joint1", Vector3f(0.0f, 2.0f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitY_, 0.0f, kVecUnitZ_, 0.0f);

    // create effectors
    ResetEffectors();
    effectors_->AddEffector(tree_.get(), "effector0", "joint2", 0.5f);

    // this needs to be done after adding all the effectors
    ResetSolver(IKSolver::DLS);
    ResetRenderer();
}

void EffectorOnBone::Update(float time)
{
    float angle = 2.0 * step_ * kPi_ / kMaxStep_;
    effectors_->UpdateTarget("effector0", 0.0f + 1.0f * sin(angle), 3.0f, 0.0f + 1.0f * cos(angle));
    solver_->Solve(tree_.get(), effectors_.get(), iterationCount_);
    step_ = (step_ + 1) % kMaxStep_;
}