#ifndef HAND_IK_H_
#define HAND_IK_H_

#include <vector>
#include "glm/glm.hpp"
#include "IKProgram.h"
#include "IKSolverDLS.h"

class HandIK : public IKProgram
{
public:
    DUMMY_DEFAULT_CONSTRUCTOR_AND_DESTRUCTOR(HandIK)

    HandIK(Data* data);

    void Update(float time) override;
private:
    typedef std::vector<int>       VecInt;
    typedef std::pair<int, int>    IntInt;
    typedef std::vector<IntInt>    VecIntInt;
    typedef std::vector<glm::vec3> VecVec3;
    typedef std::vector<VecVec3>   VecVecVec3;
    typedef std::vector<Eigen::AngleAxisf> VecAA;
    typedef std::vector<VecAA> VecVecAA;

    void ReadSkeleton(const char* skeFile, VecVec3* skePos, VecVec3* skeRots,
                      VecIntInt* skeLinks, VecInt* skeDofs, VecInt* skeEffectors);
    void ReadMotion(const char* motionFile, VecVecVec3* motion);

    // Helper function to (recursively) add joints to the tree.
    // Finds the parent and adds it before adding the current joint.
    void AddJoint(int joint, const VecVec3& skePos, const VecVec3& skeRots,
                  const VecIntInt& skeLinks, const VecInt& skeDofs);

    void WriteTree(const char* fileName);
    void WriteMotion(const char* skeFile, const VecVec3& rootPos,
        const VecVecAA& skeRots);
    float NormalizeAngle(float value, float minValue, float maxValue);

    VecVecVec3 motion_;
    std::vector<float> errors_;

    VecVec3 skePos_;
    VecVec3 skeRots_;
    VecIntInt skeLinks_;
    VecInt skeDofs_;
    VecInt skeEffectors_;
};

#endif