#include "IKProgram1.h"
#include "Eigen/Dense"

using namespace std;
using namespace Eigen;

/*
 * Default constructor
 */
IKProgram1::IKProgram1()
{
}

/*
 * Creates an IK program with the specified name
 */
IKProgram1::IKProgram1(const string &name, shared_ptr<IKRenderer> renderer, int stepsPerSecond, int fps) :
    IKProgram(name, renderer, stepsPerSecond, fps)
{
    // build the tree
    tree_.reset(new Tree());
    tree_->AddBallRoot("0", 0.08f, true, Vector3f(0.0f, -0.5f, 0.0f), Vector3f::UnitX(), 0.0f,
        Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("1", 0.08f, false, "0", Vector3f(0.0f, 0.9f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("2", 0.08f, false, "0", Vector3f(0.0f, 0.9f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("3", 0.08f, false, "1", Vector3f(0.5f, 0.6f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("4", 0.08f, false, "2", Vector3f(-0.5f, 0.6f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("5", 0.08f, false, "3", Vector3f(0.2f, 0.3f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("6", 0.08f, false, "4", Vector3f(-0.3f, 0.5f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("7", 0.08f, false, "5", Vector3f(0.0f, 0.7f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);
    tree_->AddBallJoint("8", 0.08f, false, "6", Vector3f(0.0f, 0.4f, 0.0f), Vector3f::UnitX(),
        0.0f, Vector3f::UnitY(), 0.0f, Vector3f::UnitZ(), 0.0f);

    // create the ik solver and give it the tree
    solver_.reset(new IKSolver(tree_));
    // create effectors
    solver_->AddEffector("effector0", "7", 1.0f);
    solver_->AddEffector("effector1", "8", 1.0f);
    solver_->NoMoreEffector();

    renderer_->InitBatches(tree_);
}

/*
 * Updates the program's state
 * To be called each animation frame
 */
void IKProgram1::Update(float time)
{
    // compute global data
    tree_->ComputeGlobalData();

    // set new target positions
    solver_->UpdateTarget("effector0", 0.6f + 1.5f * sin(time), 1.2f + 0.2f * sin(time),
        0.1f + 0.2f * sin(time));
    solver_->UpdateTarget("effector1", -0.7f + 0.4f * sin(time), 1.8f + 0.3f * sin(time),
        0.1f + 0.2f * sin(time));

    // do IK
    //solver_->Solve(1);
}

/*
 * Draws the scene
 * To be called each rendering frame
 */
void IKProgram1::Draw() const
{
    // TODO
    renderer_->SetCameraLookAt(0.0f, 2.0f, 5.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
    renderer_->TransformLights();
    renderer_->DrawTree(tree_);
    renderer_->DrawTargets(solver_);
    renderer_->DrawGlobalAxes(1000.0f);
}

/*
 * Destructor
 */
IKProgram1::~IKProgram1()
{
}