#include "MultipleJointTypes.h"
#include "Eigen/Dense"

using namespace std;
using namespace Eigen;

MultipleJointTypes::MultipleJointTypes() {}
MultipleJointTypes::~MultipleJointTypes() {}

MultipleJointTypes::MultipleJointTypes(Data *data) :
    IKProgram(data)
{
    name_ = "Multiple joint types";
    description_ = "This IK program tests a tree with multiple joint types and two effectors. The targets move in"
        "cyclic motions and sometimes become unreachable (to test the stability of the solver in singular cases).";

    // build the tree
    ResetTree();
    tree_->AddJoint("joint0", 0.08f, "", Vector3f(0.0f, 0.5f, 0.0f),
        kVecUnitX_, 0.0, kVecUnitY_, 0.0f, kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint1", 0.08f, "joint0", Vector3f(0.0f, 0.9f, 0.0f), kVecUnitX_, 0.0);
    tree_->AddJoint("joint2", 0.08f, "joint0", Vector3f(0.0f, 0.9f, 0.0f), kVecUnitZ_, 0.0);
    tree_->AddJoint("joint3", 0.08f, "joint1", Vector3f(0.0f, 0.6f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint4", 0.08f, "joint2", Vector3f(0.0f, 0.6f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint5", 0.08f, "joint3", Vector3f(0.0f, 0.3f, 0.0f), kVecUnitX_, 0.0f);
    tree_->AddJoint("joint6", 0.08f, "joint4", Vector3f(0.0f, 0.5f, 0.0f), kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint7", 0.08f, "joint5", Vector3f(0.0f, 0.7f, 0.0f), kVecUnitX_, 0.0f);
    tree_->AddJoint("joint8", 0.08f, "joint6", Vector3f(0.0f, 0.4f, 0.0f), kVecUnitX_, 0.0f);

    // create effectors
    ResetEffectors();
    effectors_->AddEffector(tree_.get(), "effector0", "joint7", 1.0f);
    effectors_->AddEffector(tree_.get(), "effector1", "joint8", 1.0f);

    ResetSolver(IKSolver::DLS);
    ResetRenderer();
}

void MultipleJointTypes::Update(float time)
{
    float angle = 2.0f * step_ * kPi_ / kMaxStep_;
    step_ = (step_ + 1) % kMaxStep_;

    effectors_->UpdateTarget("effector0",
        2.0f + 0.6f * sin(angle),
        1.7f + 0.6f * sin(2.0f * angle),
        0.3f + 0.7f * sin(3.0f * angle));
    effectors_->UpdateTarget("effector1",
        -0.7f + 0.7f * sin(angle),
         1.3f + 0.5f * sin(2.0f * angle),
        -0.2f + 0.6f * sin(3.0f * angle));

    solver_->Solve(tree_.get(), effectors_.get(), iterationCount_);
}