#include "ThreeHingeJoints.h"
#include "Eigen/Dense"

using namespace std;
using namespace Eigen;

ThreeHingeJoints::ThreeHingeJoints()
{
}

ThreeHingeJoints::~ThreeHingeJoints()
{
}

ThreeHingeJoints::ThreeHingeJoints(Data *data) :
    IKProgram(data),
    sign_(-1.0f)
{
    name_ = "Three hinge joints";
    description_ = "This IK program tests a simple tree with three hinge joints.";

    // build the tree
    ResetTree();
    tree_->AddJoint("joint0", 0.08f, "", Vector3f(0.0f, 0.5f, 0.0f), kVecUnitX_, 0.0f);
    tree_->AddJoint("joint1", 0.08f, "joint0", Vector3f(0.0f, 2.0f, 0.0f), kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint2", 0.08f, "joint1", Vector3f(2.0f, 0.0f, 0.0f), kVecUnitZ_, 0.0f);

    // create effectors
    ResetEffectors();
    effectors_->AddEffector(tree_.get(), "effector0", "joint2", 1.0f);

    ResetSolver(IKSolver::DLS);
    ResetRenderer();
}

void ThreeHingeJoints::Update(float time)
{
    float angle = 90.0f + 2.0f * sign_ * step_ * kPi_ / kMaxStep_;

    effectors_->UpdateTarget("effector0", 0.0f + 2.0f * sin(angle), 2.5f + 2.0f * cos(angle), 0.0f + 4.0 * cos(angle));
    solver_->Solve(tree_.get(), effectors_.get(), iterationCount_);

    if (step_ >= kMaxStep_) {
        sign_ = -sign_;
        step_ = 0;
    }
    step_++;
}