#include "TwoBallJoints.h"
#include "Eigen/Dense"

using namespace std;
using namespace Eigen;

TwoBallJoints::TwoBallJoints(Data *data) :
    IKProgram(data)
{
    name_ = "Two ball joints";
    description_ = "This IK program tests a simple tree with two ball joints.";

    // build the tree
    ResetTree();
    tree_->AddJoint("joint0", 0.08f, "", Vector3f(0.0f, 0.5f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitY_, 0.0f, kVecUnitZ_, 0.0f);
    tree_->AddJoint("joint1", 0.08f, "joint0", Vector3f(2.0f, 0.0f, 0.0f),
        kVecUnitX_, 0.0f, kVecUnitY_, 0.0f, kVecUnitZ_, 0.0f);
    // create effectors
    ResetEffectors();
    effectors_->AddEffector(tree_.get(), "effector0", "joint1", 1.0f);

    ResetSolver(IKSolver::DLS);
    ResetRenderer();
}

void TwoBallJoints::Update(float time)
{
    float angle = 2.0f * step_ * kPi_ / kMaxStep_;
    effectors_->UpdateTarget("effector0", 0.0f + 1.0f * sin(angle), 2.5f, 0.0f + 1.0f * cos(angle));
    solver_->Solve(tree_.get(), effectors_.get(), iterationCount_);
    step_ = (step_ + 1) % kMaxStep_;
}