#ifndef RENDERER_H__
#define RENDERER_H__

#include <memory>
#include <GLEW/glew.h>
#include <GLM/glm.hpp>
#include <Freeglut/freeglut.h>
#include "Camera3D.h"
#include "EffectorCollection.h"

struct Light {
    glm::vec4 ambient;
    glm::vec4 diffuse;
    glm::vec4 specular;
    glm::vec4 position;
};

class Renderer {
public:
    enum Renderable {
        GLOBAL_AXES,
        FLOOR,
        JOINT,
        BONE,
        TARGET,
        TARGET_LINE,
        LOCAL_FRAME,
        STUFF_COUNT
    };

    Renderer();
    Renderer(Camera3D *camera);
    virtual ~Renderer();

    void SetCameraLookAt(float eyeX, float eyeY, float eyeZ,
                         float centerX, float centerY, float centerZ,
                         float upX, float upY, float upZ);
    void DrawGlobalAxes(float length) const;
    void DrawFloor(float size) const;

    void DrawTree(const Tree* tree) const;
    void DrawTargets(const EffectorCollection* tree, bool drawLines = true) const;
    void SetColor(Renderable which, const glm::vec4& color);

    void Enable(Renderable which);
    void Disable(Renderable which);
    bool IsEnable(Renderable which) const;

protected:
    void DrawCylinder(Eigen::Vector3f axis, float radius) const;
    void CreateFloorTexture();
    void SetUpLights();
    void CreateShapes();
    void SetDefaultEnable();
    void SetDefaultColor();

    enum Shape {
        SPHERE,
        CYLINDER,
        SHAPE_COUNT
    };

    glm::vec4 color_[STUFF_COUNT];
    bool isEnabled_[STUFF_COUNT];
    GLuint shape_[SHAPE_COUNT];

    Camera3D* camera_;
    Light light_;

    GLuint floorTexture_;
};

#endif    // RENDERER_H__